@@ -6,6 +6,42 @@
 ###
 
 
+### TODO: Move this to S4Vectors (or BiocBaseUtils).
+load_package_gracefully <- function(package, ...)
+{
+    if (!requireNamespace(package, quietly=TRUE))
+        stop("Could not load package ", package, ". Is it installed?\n\n  ",
+             wmsg("Note that ", ..., " requires the ", package, " package. ",
+                  "Please install it with:"),
+             "\n\n    BiocManager::install(\"", package, "\")")
+}
+
+### If 'collapse' is FALSE (the default), returns a logical vector parallel
+### to 'x'. Otherwise, returns a single logical TRUE or FALSE.
+### TODO: Move this to Biostrings (put it in same file as replaceAmbiguities).
+hasAmbiguities <- function(x, other.than.N=FALSE, collapse=FALSE)
+{
+    stopifnot(is(x, "DNAStringSet"),
+              isTRUEorFALSE(other.than.N),
+              isTRUEorFALSE(collapse))
+    ambig_letters <- setdiff(names(IUPAC_CODE_MAP), DNA_BASES)
+    if (other.than.N)
+        ambig_letters <- setdiff(ambig_letters, "N")
+    af <- alphabetFrequency(x, collapse=collapse)
+    if (collapse) {
+        ## 'af' is a named integer vector.
+        stopifnot(all(ambig_letters %in% names(af)))
+        ambig_count <- sum(af[ambig_letters])
+    } else {
+        ## 'af' is an integer matrix with 1 row per sequence in 'x'.
+        stopifnot(all(ambig_letters %in% colnames(af)))
+        ambig_count <- rowSums(af[ , ambig_letters, drop=FALSE])
+        if (!is.null(names(x)))
+            names(ambig_count) <- names(x)
+    }
+    ambig_count != 0L
+}
+
 ### Returns TRUE if 'assembly_accession' is a GenBank accession, or FALSE
 ### if it's a RefSeq accession, or an error if it's none.
 is_GenBank_accession <- function(assembly_accession)
@@ -43,7 +79,7 @@ drop_rows_with_NA_accns <- function(chrominfo, accession_col)
 ###   reconstructed <- paste0(prefix_suffix[ , 1], prefix_suffix[ , 2])
 ###   stopifnot(identical(x, reconstructed))
 .split_suffix <- function(x, character_class)
-{   
+{
     pattern <- sprintf("^.*[^%s]([%s]*)$", character_class, character_class)
     suffix <- sub(pattern, "\\1", x, perl=TRUE)
     prefix <- substr(x, 1L, nchar(x) - nchar(suffix))
@@ -95,11 +131,19 @@ abbreviate_organism_name <- function(organism)
 
 check_pkg_maintainer <- function(pkg_maintainer)
 {
+    if (!(isSingleString(pkg_maintainer) && nzchar(pkg_maintainer)))
+        stop(wmsg("'pkg_maintainer' must be a single (non-empty) string"))
     pattern <- "\\<[A-Z0-9._%+-]+@[A-Z0-9.-]+\\.[A-Z]{2,}\\>"
     if (!grepl(pattern, pkg_maintainer, ignore.case=TRUE))
         stop(wmsg("please provide a valid email address"))
 }
 
+make_pkgname <- function(abbr_organism, provider, genome)
+{
+    part4 <- gsub("[^0-9a-zA-Z.]", "", genome)
+    paste("BSgenome", abbr_organism, provider, part4, sep=".")
+}
+
 organism2biocview <- function(organism)
 {
     parts <- strsplit(organism, " +")[[1]]
@@ -110,17 +154,12 @@ organism2biocview <- function(organism)
 
 build_Rexpr_as_string <- function(seqnames)
 {
-    if (length(seqnames) == 0)
+    stopifnot(is.character(seqnames))
+    if (length(seqnames) == 0L)
         return("character(0)")
     paste0('c', '(', paste0('"', seqnames, '"', collapse=","), ')')
 }
 
-move_file_to_datapkg <- function(filepath, pkg_dir)
-{
-    to <- file.path(pkg_dir, "inst", "extdata", basename(filepath))
-    file.rename(filepath, to)
-}
-
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Helper functions used by .get_circ_seqs_from_NCBI() and
@@ -133,9 +172,10 @@ check_circ_seqs <- function(circ_seqs)
         return()
     if (!is.character(circ_seqs))
         stop(wmsg("'circ_seqs' must be NULL or a character vector"))
+    ## "primary key" constraints (see GenomeInfoDb:::is_primary_key)
     if (anyNA(circ_seqs))
         stop(wmsg("'circ_seqs' cannot contain NA's"))
-    if ("" %in% circ_seqs)
+    if (!all(nzchar(circ_seqs)))
         stop(wmsg("'circ_seqs' cannot contain empty strings"))
     if (anyDuplicated(circ_seqs))
         stop(wmsg("'circ_seqs' cannot contain duplicate values"))
@@ -151,7 +191,7 @@ get_circ_seqs_for_registered_assembly_or_genome <-
     if (setequal(circ_seqs, known_circ_seqs))
         return(circ_seqs)
     msg <- c("This ", what, " is registered in the GenomeInfoDb package ")
-    if (length(known_circ_seqs) == 0) {
+    if (length(known_circ_seqs) == 0L) {
         msg <- c(msg, "and it has no known circular sequences.")
     } else {
         in1string <- paste0("\"", known_circ_seqs, "\"", collapse=", ")
@@ -167,7 +207,7 @@ get_circ_seqs_for_registered_assembly_or_genome <-
 
 .assembly_has_no_assembled_molecules <- function(circ_seqs, what)
 {
-    if (length(circ_seqs) == 0)
+    if (length(circ_seqs) == 0L)
         return(character(0))
     stop(wmsg("This ", what, " contains no assembled molecules ",
               "so cannot have circular sequences."))
